@@ -32,8 +32,8 @@ if 'PYTHONWARNINGS' not in os.environ:
 warnings.filterwarnings('ignore', "'U' mode is deprecated",
                         DeprecationWarning, module='docutils.io')
 
-__version__ = '2.4.3'
-__released__ = '2.4.3'  # used when Sphinx builds its own docs
+__version__ = '3.2.1'
+__released__ = '3.2.1'  # used when Sphinx builds its own docs
 
 #: Version info for better programmatic use.
 #:
@@ -43,7 +43,7 @@ __released__ = '2.4.3'  # used when Sphinx builds its own docs
 #:
 #: .. versionadded:: 1.2
 #:    Before version 1.2, check the string ``sphinx.__version__``.
-version_info = (2, 4, 3, 'final', 0)
+version_info = (3, 2, 1, 'final', 0)
 
 package_dir = path.abspath(path.dirname(__file__))
 
@@ -56,8 +56,9 @@ if __version__.endswith('+'):
     __version__ = __version__[:-1]  # remove '+' for PEP-440 version spec.
     try:
         ret = subprocess.run(['git', 'show', '-s', '--pretty=format:%h'],
-                             stdout=PIPE, stderr=PIPE, encoding='ascii')
+                             cwd=package_dir,
+                             stdout=PIPE, stderr=PIPE)
         if ret.stdout:
-            __display_version__ += '/' + ret.stdout.strip()
+            __display_version__ += '/' + ret.stdout.decode('ascii').strip()
     except Exception:
         pass
