@@ -23,7 +23,11 @@ deprecated_alias('sphinx.builders.qthelp',
                      'render_file': render_file,
                      'QtHelpBuilder': QtHelpBuilder,
                  },
-                 RemovedInSphinx40Warning)
+                 RemovedInSphinx40Warning,
+                 {
+                     'render_file': 'sphinxcontrib.qthelp.render_file',
+                     'QtHelpBuilder': 'sphinxcontrib.qthelp.QtHelpBuilder',
+                 })
 
 
 def setup(app: Sphinx) -> Dict[str, Any]:
