@@ -18,7 +18,9 @@ from docutils.parsers.rst import directives, roles
 
 from sphinx import addnodes
 from sphinx.addnodes import desc_signature
-from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
+from sphinx.deprecation import (
+    RemovedInSphinx40Warning, RemovedInSphinx50Warning, deprecated_alias
+)
 from sphinx.util import docutils
 from sphinx.util.docfields import DocFieldTransformer, Field, TypedField
 from sphinx.util.docutils import SphinxDirective
@@ -34,7 +36,7 @@ nl_escape_re = re.compile(r'\\\n')
 strip_backslash_re = re.compile(r'\\(.)')
 
 
-def optional_int(argument):
+def optional_int(argument: str) -> int:
     """
     Check for an integer argument or None value; raise ``ValueError`` if not.
     """
@@ -89,12 +91,13 @@ class ObjectDescription(SphinxDirective):
         """
         Retrieve the signatures to document from the directive arguments.  By
         default, signatures are given as arguments, one per line.
-
-        Backslash-escaping of newlines is supported.
         """
         lines = nl_escape_re.sub('', self.arguments[0]).split('\n')
-        # remove backslashes to support (dummy) escapes; helps Vim highlighting
-        return [strip_backslash_re.sub(r'\1', line.strip()) for line in lines]
+        if self.config.strip_signature_backslash:
+            # remove backslashes to support (dummy) escapes; helps Vim highlighting
+            return [strip_backslash_re.sub(r'\1', line.strip()) for line in lines]
+        else:
+            return [line.strip() for line in lines]
 
     def handle_signature(self, sig: str, signode: desc_signature) -> Any:
         """
@@ -123,6 +126,15 @@ class ObjectDescription(SphinxDirective):
         """
         pass
 
+    def transform_content(self, contentnode: addnodes.desc_content) -> None:
+        """
+        Called after creating the content through nested parsing,
+        but before the ``object-description-transform`` event is emitted,
+        and before the info-fields are transformed.
+        Can be used to manipulate the content.
+        """
+        pass
+
     def after_content(self) -> None:
         """
         Called after parsing content. Used to reset information about the
@@ -160,6 +172,8 @@ class ObjectDescription(SphinxDirective):
         # 'desctype' is a backwards compatible attribute
         node['objtype'] = node['desctype'] = self.objtype
         node['noindex'] = noindex = ('noindex' in self.options)
+        if self.domain:
+            node['classes'].append(self.domain)
 
         self.names = []  # type: List[Any]
         signatures = self.get_signatures()
@@ -167,7 +181,7 @@ class ObjectDescription(SphinxDirective):
             # add a signature node for each signature in the current unit
             # and add a reference target for it
             signode = addnodes.desc_signature(sig, '')
-            signode['first'] = False
+            self.set_source_info(signode)
             node.append(signode)
             try:
                 # name can also be a tuple, e.g. (classname, objname);
@@ -193,6 +207,7 @@ class ObjectDescription(SphinxDirective):
             self.env.temp_data['object'] = self.names[0]
         self.before_content()
         self.state.nested_parse(self.content, self.content_offset, contentnode)
+        self.transform_content(contentnode)
         self.env.app.emit('object-description-transform',
                           self.domain, self.objtype, contentnode)
         DocFieldTransformer(self).transform_all(contentnode)
@@ -283,14 +298,39 @@ deprecated_alias('sphinx.directives',
                      'Figure': Figure,
                      'Meta': Meta,
                  },
-                 RemovedInSphinx40Warning)
+                 RemovedInSphinx40Warning,
+                 {
+                     'Highlight': 'sphinx.directives.code.Highlight',
+                     'CodeBlock': 'sphinx.directives.code.CodeBlock',
+                     'LiteralInclude': 'sphinx.directives.code.LiteralInclude',
+                     'TocTree': 'sphinx.directives.other.TocTree',
+                     'Author': 'sphinx.directives.other.Author',
+                     'Index': 'sphinx.directives.other.IndexDirective',
+                     'VersionChange': 'sphinx.directives.other.VersionChange',
+                     'SeeAlso': 'sphinx.directives.other.SeeAlso',
+                     'TabularColumns': 'sphinx.directives.other.TabularColumns',
+                     'Centered': 'sphinx.directives.other.Centered',
+                     'Acks': 'sphinx.directives.other.Acks',
+                     'HList': 'sphinx.directives.other.HList',
+                     'Only': 'sphinx.directives.other.Only',
+                     'Include': 'sphinx.directives.other.Include',
+                     'Class': 'sphinx.directives.other.Class',
+                     'Figure': 'sphinx.directives.patches.Figure',
+                     'Meta': 'sphinx.directives.patches.Meta',
+                 })
 
-
-# backwards compatible old name (will be marked deprecated in 3.0)
-DescDirective = ObjectDescription
+deprecated_alias('sphinx.directives',
+                 {
+                     'DescDirective': ObjectDescription,
+                 },
+                 RemovedInSphinx50Warning,
+                 {
+                     'DescDirective': 'sphinx.directives.ObjectDescription',
+                 })
 
 
 def setup(app: "Sphinx") -> Dict[str, Any]:
+    app.add_config_value("strip_signature_backslash", False, 'env')
     directives.register_directive('default-role', DefaultRole)
     directives.register_directive('default-domain', DefaultDomain)
     directives.register_directive('describe', ObjectDescription)
