@@ -1,16 +1,16 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.search.zh
     ~~~~~~~~~~~~~~~~
 
     Chinese search language: includes routine to split words.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 import re
+from typing import Dict, List
 
 from sphinx.search import SearchLanguage
 from sphinx.util.stemmer import get_stemmer
@@ -21,11 +21,7 @@ try:
 except ImportError:
     JIEBA = False
 
-if False:
-    # For type annotation
-    from typing import Dict, List  # NOQA
-
-english_stopwords = set(u"""
+english_stopwords = set("""
 a  and  are  as  at
 be  but  by
 for
@@ -234,10 +230,9 @@ class SearchChinese(SearchLanguage):
     js_stemmer_code = js_porter_stemmer
     stopwords = english_stopwords
     latin1_letters = re.compile(r'[a-zA-Z0-9_]+')
-    latin_terms = []  # type: List[unicode]
+    latin_terms = []  # type: List[str]
 
-    def init(self, options):
-        # type: (Dict) -> None
+    def init(self, options: Dict) -> None:
         if JIEBA:
             dict_path = options.get('dict')
             if dict_path and os.path.isfile(dict_path):
@@ -245,9 +240,8 @@ class SearchChinese(SearchLanguage):
 
         self.stemmer = get_stemmer()
 
-    def split(self, input):
-        # type: (unicode) -> List[unicode]
-        chinese = []  # type: List[unicode]
+    def split(self, input: str) -> List[str]:
+        chinese = []  # type: List[str]
         if JIEBA:
             chinese = list(jieba.cut_for_search(input))
 
@@ -256,13 +250,10 @@ class SearchChinese(SearchLanguage):
         self.latin_terms.extend(latin1)
         return chinese + latin1
 
-    def word_filter(self, stemmed_word):
-        # type: (unicode) -> bool
+    def word_filter(self, stemmed_word: str) -> bool:
         return len(stemmed_word) > 1
 
-    def stem(self, word):
-        # type: (unicode) -> unicode
-
+    def stem(self, word: str) -> str:
         # Don't stem Latin words that are long enough to be relevant for search
         # if not stemmed, but would be too short after being stemmed
         # avoids some issues with acronyms
