@@ -10,16 +10,12 @@
 
 import base64
 import imghdr
-import warnings
 from collections import OrderedDict
-from io import BytesIO
 from os import path
-from typing import IO, NamedTuple, Tuple
+from typing import IO, NamedTuple, Optional, Tuple
 
 import imagesize
 
-from sphinx.deprecation import RemovedInSphinx30Warning
-
 try:
     from PIL import Image
 except ImportError:
@@ -40,7 +36,7 @@ DataURI = NamedTuple('DataURI', [('mimetype', str),
                                  ('data', bytes)])
 
 
-def get_image_size(filename: str) -> Tuple[int, int]:
+def get_image_size(filename: str) -> Optional[Tuple[int, int]]:
     try:
         size = imagesize.get(filename)
         if size[0] == -1:
@@ -49,36 +45,26 @@ def get_image_size(filename: str) -> Tuple[int, int]:
             size = (int(size[0]), int(size[1]))
 
         if size is None and Image:  # fallback to Pillow
-            im = Image.open(filename)
-            size = im.size
-            try:
-                im.fp.close()
-            except Exception:
-                pass
+            with Image.open(filename) as im:
+                size = im.size
 
         return size
     except Exception:
         return None
 
 
-def guess_mimetype_for_stream(stream: IO, default: str = None) -> str:
-    imgtype = imghdr.what(stream)  # type: ignore
+def guess_mimetype_for_stream(stream: IO, default: Optional[str] = None) -> Optional[str]:
+    imgtype = imghdr.what(stream)
     if imgtype:
         return 'image/' + imgtype
     else:
         return default
 
 
-def guess_mimetype(filename: str = '', content: bytes = None, default: str = None) -> str:
+def guess_mimetype(filename: str = '', default: Optional[str] = None) -> Optional[str]:
     _, ext = path.splitext(filename.lower())
     if ext in mime_suffixes:
         return mime_suffixes[ext]
-    elif content:
-        # TODO: When the content argument is removed, make filename a required
-        # argument.
-        warnings.warn('The content argument of guess_mimetype() is deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-        return guess_mimetype_for_stream(BytesIO(content), default=default)
     elif path.exists(filename):
         with open(filename, 'rb') as f:
             return guess_mimetype_for_stream(f, default=default)
@@ -86,7 +72,7 @@ def guess_mimetype(filename: str = '', content: bytes = None, default: str = Non
     return default
 
 
-def get_image_extension(mimetype: str) -> str:
+def get_image_extension(mimetype: str) -> Optional[str]:
     for ext, _mimetype in mime_suffixes.items():
         if mimetype == _mimetype:
             return ext
@@ -94,7 +80,7 @@ def get_image_extension(mimetype: str) -> str:
     return None
 
 
-def parse_data_uri(uri: str) -> DataURI:
+def parse_data_uri(uri: str) -> Optional[DataURI]:
     if not uri.startswith('data:'):
         return None
 
@@ -115,7 +101,7 @@ def parse_data_uri(uri: str) -> DataURI:
     return DataURI(mimetype, charset, image_data)
 
 
-def test_svg(h: bytes, f: IO) -> str:
+def test_svg(h: bytes, f: IO) -> Optional[str]:
     """An additional imghdr library helper; test the header is SVG's or not."""
     try:
         if '<svg' in h.decode().lower():
