@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # set this by test
 # import os
 # import sys
@@ -11,11 +10,9 @@ from sphinx.writers.latex import LaTeXTranslator
 from sphinx.writers.manpage import ManualPageTranslator
 from sphinx.writers.texinfo import TexinfoTranslator
 from sphinx.writers.text import TextTranslator
-from sphinx.writers.websupport import WebSupportTranslator
 
 
 project = 'test'
-master_doc = 'index'
 
 
 class ConfHTMLTranslator(HTMLTranslator):
@@ -54,10 +51,6 @@ class ConfTextTranslator(TextTranslator):
     pass
 
 
-class ConfWebSupportTranslator(WebSupportTranslator):
-    pass
-
-
 class ConfXMLTranslator(XMLTranslator):
     pass
 
@@ -76,6 +69,5 @@ def setup(app):
     app.set_translator('man', ConfManualPageTranslator)
     app.set_translator('texinfo', ConfTexinfoTranslator)
     app.set_translator('text', ConfTextTranslator)
-    app.set_translator('websupport', ConfWebSupportTranslator)
     app.set_translator('xml', ConfXMLTranslator)
     app.set_translator('pseudoxml', ConfPseudoXMLTranslator)
