@@ -1,9 +1,6 @@
-# -*- coding: utf-8 -*-
-
 import os
 import sys
 
 sys.path.insert(0, os.path.dirname(os.path.abspath('.')))
 
 project = 'test'
-master_doc = 'index'
