@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-
-master_doc = 'index'
 exclude_patterns = ['_build']
 
 rst_epilog = '''
