@@ -11,8 +11,8 @@
 import os
 
 import alabaster
-import pytest
 
+import pytest
 from sphinx.theming import ThemeError
 
 
@@ -74,18 +74,18 @@ def test_js_source(app, status, warning):
 
     app.builder.build(['contents'])
 
-    v = '3.4.1'
+    v = '3.5.1'
     msg = 'jquery.js version does not match to {v}'.format(v=v)
-    jquery_min = (app.outdir / '_static' / 'jquery.js').text()
+    jquery_min = (app.outdir / '_static' / 'jquery.js').read_text()
     assert 'jQuery v{v}'.format(v=v) in jquery_min, msg
-    jquery_src = (app.outdir / '_static' / 'jquery-{v}.js'.format(v=v)).text()
+    jquery_src = (app.outdir / '_static' / 'jquery-{v}.js'.format(v=v)).read_text()
     assert 'jQuery JavaScript Library v{v}'.format(v=v) in jquery_src, msg
 
     v = '1.3.1'
     msg = 'underscore.js version does not match to {v}'.format(v=v)
-    underscore_min = (app.outdir / '_static' / 'underscore.js').text()
+    underscore_min = (app.outdir / '_static' / 'underscore.js').read_text()
     assert 'Underscore.js {v}'.format(v=v) in underscore_min, msg
-    underscore_src = (app.outdir / '_static' / 'underscore-{v}.js'.format(v=v)).text()
+    underscore_src = (app.outdir / '_static' / 'underscore-{v}.js'.format(v=v)).read_text()
     assert 'Underscore.js {v}'.format(v=v) in underscore_src, msg
 
 
@@ -108,21 +108,37 @@ def test_staticfiles(app, status, warning):
     app.build()
     assert (app.outdir / '_static' / 'staticimg.png').exists()
     assert (app.outdir / '_static' / 'statictmpl.html').exists()
-    assert (app.outdir / '_static' / 'statictmpl.html').text() == (
+    assert (app.outdir / '_static' / 'statictmpl.html').read_text() == (
         '<!-- testing static templates -->\n'
         '<html><project>Python</project></html>'
     )
 
-    result = (app.outdir / 'index.html').text()
+    result = (app.outdir / 'index.html').read_text()
     assert '<meta name="testopt" content="optdefault" />' in result
 
 
+@pytest.mark.sphinx(testroot='theming',
+                    confoverrides={'html_theme': 'test-theme'})
+def test_dark_style(app, status, warning):
+    style = app.builder.dark_highlighter.formatter_args.get('style')
+    assert style.__name__ == 'MonokaiStyle'
+
+    app.build()
+    assert (app.outdir / '_static' / 'pygments_dark.css').exists()
+
+    result = (app.outdir / 'index.html').read_text()
+    assert '<link rel="stylesheet" href="_static/pygments.css" type="text/css" />' in result
+    assert ('<link id="pygments_dark_css" media="(prefers-color-scheme: dark)" '
+            'rel="stylesheet" type="text/css" '
+            'href="_static/pygments_dark.css" />') in result
+
+
 @pytest.mark.sphinx(testroot='theming')
 def test_theme_sidebars(app, status, warning):
     app.build()
 
     # test-theme specifies globaltoc and searchbox as default sidebars
-    result = (app.outdir / 'index.html').text(encoding='utf8')
+    result = (app.outdir / 'index.html').read_text()
     assert '<h3><a href="#">Table of Contents</a></h3>' in result
     assert '<h3>Related Topics</h3>' not in result
     assert '<h3>This Page</h3>' not in result
